/**********************************************************************
 *
 * $Id: sdp_listener.c,v 1.3 2000/05/04 19:44:40 htran Exp $
 *
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 **********************************************************************/

#include <pthread.h>
#include <stdlib.h>
#include <string.h>
#include <gtk/gtk.h>
#include <netinet/in.h>
#include <sys/socket.h>
#include <sys/types.h>
#include "sdp_listener.h"
#include "lists.h"
#include "../mim/start_msock.h"
#include "../mim/uTypes.h"

#define BUFFERSIZE 4096

/* auxilliary functions */
int parse_sap_packet(char *data, char **key, char **name);

void *sdp_listener(){
  
  typedef struct {
    uint32_t sapHdr;
    uint32_t sapSrc;
    char sapData[1];
  }sapPacket_t;
  

  extern GtkWidget *mpegVideoCLIST;
  extern GtkWidget *otherVideoCLIST;
  extern node *mpegListHead,*mpegListTail;
  extern node *otherListHead, *otherListTail;
  
  int returnVal;
  int sock, addressSize;
  sapPacket_t *s=NULL;
  struct sockaddr_in sourceAddress;
  
  char *data=(char *)malloc(BUFFERSIZE);
  char *duplicate=NULL;

  char *items[]={"NULL"};
  
  if( (sock=start_msock(SAP_ADDR,SAP_PORT,1,127,0)) <0 ){
    perror("start_msock(SAP_ADDR)");
    fprintf(stderr,"Could not start sdp listener.\n");
    exit(-1);
  }
  fprintf(stderr,"sdp Listener started\n");
  addressSize=sizeof(struct sockaddr_in);
  
  while(1){    
	 node *thisNode=NULL;
	 char *key, *name;
	 
	 memset(data,0,BUFFERSIZE);
    returnVal=recvfrom(sock,(char *)data,BUFFERSIZE,0,
							  (struct sockaddr *)&sourceAddress,&addressSize);
    if(returnVal<0){
      perror("recvfrom()");
      break;
    }
    s=(sapPacket_t *)data;
	 duplicate=(char *)malloc(returnVal);
    strcpy(duplicate,s->sapData);
	 
	 /* parse the packet for key and name */
	 key=(char *)strtok(duplicate,"\n");
	 key=(char *)strtok(NULL,"\n");
	 name=(char *)strtok(NULL,"\n");
	 name=(char *)strtok(name,"=");
	 name=(char *)strtok(NULL,"=");
	 
	 if(key!=NULL && name!=NULL && s->sapData!=NULL){
		thisNode=new_node(key,name,s->sapData);
		gdk_threads_enter();
		if( (char *)strstr(thisNode->data, "RTP/AVP 32")!=NULL ){
		  if( find_node(mpegListHead, thisNode->key)==NULL ){
			 //fprintf(stderr,"%s <- ADD MPEG\n", thisNode->name);
			 items[0]=(char *)strdup(thisNode->name);
			 insert_node(&mpegListHead,&mpegListTail, &thisNode);
			 gtk_clist_append(GTK_CLIST(mpegVideoCLIST),items);
		  }
		  else
			 ;
			 //fprintf(stderr,"%s <- DROP OTHER\n", thisNode->name);
		}
		else {
		  if( find_node(otherListHead,thisNode->key)==NULL ){
			 //fprintf(stderr,"%s <- ADD OTHER\n", thisNode->name);
			 items[0]=(char *)strdup(thisNode->name);
			 insert_node(&otherListHead,&otherListTail, &thisNode);
			 gtk_clist_append(GTK_CLIST(otherVideoCLIST),items);
		  }
		  else
			 ;
		  
			 //fprintf(stderr,"%s <- DROP OTHER\n", thisNode->name);
		}
		if(thisNode!=NULL)
		  free(thisNode);
		
		gdk_threads_leave();
				
	 }	 
  }
  
  return NULL;
  
}

int parse_sap_packet(char *data, char **key, char **name){
  char *duplicate,*this_key,*this_name,*this_data;  
  duplicate=(char *)strdup(data);
  
  this_key=(char *)strtok(duplicate,"\n");
  this_key=(char *)strtok(NULL,"\n");
  this_name=(char *)strtok(NULL,"\n");
  this_name=(char *)strtok(this_name,"=");
  this_name=(char *)strtok(NULL,"=");
  this_data=(char *)strdup(data);
  *key=(char *)malloc(strlen(this_key));
  *name=(char *)malloc(strlen(this_name));
  strcpy(*key, this_key);
  strcpy(*name, this_name);
  //fprintf(stderr,"%s\n%s\n", *key, *name);
  return 1;
}



